const connections = {}
const datachannels = {}

function createConnection(target) {
    const rpc = new RTCPeerConnection(rpcConfig)

    rpc.onicecandidate = ({candidate}) => {
        if(candidate && candidate.candidate) {
            const value = {type: 'candidate', candidate}
            wire({kind: 'rpc', value, target})
        }
    }
    rpc.ontrack = ({track}) => {
        console.log(track)
    }
    rpc.onconnectionstatechange = () => {
        if(rpc.connectionState === 'failed') {
            console.log(target, 'failed, retry!')
            wire({kind: 'rpc', value: {type: 'request'}, target})
        }
    }
    rpc.ondatachannel = ({channel}) => {
        datachannels[target] = channel
        datachannels[target].onmessage = ({data}) => console.log(data)

        // for testing purposes
        const msg = `rpc established from ${target} to ${State.username}`
        datachannels[target].send(msg)
        console.log(msg)
    }

    connections[target] = rpc
    if(State.username > target) {
        datachannels[target] = rpc.createDataChannel('test')
        datachannels[target].onmessage = ({data}) => console.log(data)
        signal({kind: 'rpc', value: {type: 'request'}, source: target})
    }
}

async function handlePeerInfo({source: target, value}) {
    const rpc = connections[target]
    if(!rpc) {
        return
    }
    if(value.type === 'request') {
        const localOffer = await rpc.createOffer()
        await rpc.setLocalDescription(localOffer)
        wire({kind: 'rpc', value: localOffer, target})
    }
    else if(value.type === 'offer') {
        const remoteOffer = new RTCSessionDescription(value)
        await rpc.setRemoteDescription(remoteOffer)
        const localAnswer = await rpc.createAnswer()
        await rpc.setLocalDescription(localAnswer)
        wire({kind: 'rpc', value: localAnswer, target})
    }
    else if(value.type === 'answer') {
        const remoteAnswer = new RTCSessionDescription(value)
        await rpc.setRemoteDescription(remoteAnswer)
    }
    else if(value.type === 'candidate') {
        const candidate = new RTCIceCandidate(value.candidate)
        await rpc.addIceCandidate(candidate)
    }
}

function removeConnection(user) {
    const rpc = connections[user]
    if(rpc) {
        delete connections[user]
    }
}

addEventListener('rpc', (e) => handlePeerInfo(e.detail))
addEventListener('join', (e) => createConnection(e.detail.value))
addEventListener('load', () => doNotLog.add('rpc'))
