const Toggle = {
    view({attrs: {label, value}}) {
        const onclick = () => {
            StreamContainer[value] = !StreamContainer[value]
            updateSelfVideo()
        }
        const checked = StreamContainer[value]
        return m('label.styled',
            m('input[type=checkbox]', {checked, onclick}),
            label,
        )
    }
}
const VideoConfig = {
    get video() {
        return StreamContainer.videoOn
        && State.online.length < 10
        && params.get('v') !== '0'
        && {width: {ideal: 320}, facingMode: 'user', frameRate: 26}
    },
    get audio() {
        return StreamContainer.audioOn
        && params.get('a') !== '0'
    },
    view() {
        return [
            m(Toggle, {label: 'video', value: 'videoOn'}),
            m(Toggle, {label: 'audio', value: 'audioOn'}),
        ]
    }
}
const updateSelfVideo = async () => {
    const video = document.querySelector('video.self')

    video.srcObject.getTracks().forEach(track => {
        track.stop()
        video.srcObject.removeTrack(track)
        delete track
    })

    if(StreamContainer.videoOn || StreamContainer.audioOn) {
        await navigator.mediaDevices
        .getUserMedia(VideoConfig)
        .then(s => s.getTracks().forEach(t => video.srcObject.addTrack(t)))
        .catch(e => console.error(e))
    }
    video.srcObject = video.srcObject  // safari

    wire({kind: 'peerInfo', value: {type: 'request'}})
}
const updateOtherVideo = (target, dom) => {
    dom.srcObject = new MediaStream()
    let rpc = null

    const stopRpc = () => {
        rpc && rpc.close()
        dom.srcObject.getTracks().forEach(track => {
            track.stop()
            dom.srcObject.removeTrack(track)
            delete track
        })
    }

    const resetRpc = () => {
        stopRpc()
        rpc = new RTCPeerConnection(rpcConfig)
        document.querySelector('video.self').srcObject.getTracks()
            .forEach(t => rpc.addTrack(t))
    }
}
const Video = {
    setUp: (username) => ({dom}) => {
        dom.username = username
        dom.srcObject = new MediaStream()
        if(username === State.username) {
            dom.classList.add('self')
            dom.muted = true
            updateSelfVideo()
        }
        else {
            updateOtherVideo(username, dom)
        }
    },
    tearDown: (username) => ({dom}) => {
        removeEventListener('peerInfo', dom.listener)

        dom.srcObject.getTracks().forEach(track => {
            track.stop()
            dom.srcObject.removeTrack(track)
            delete track
        })
    },
    view({attrs: {username}}) {
        const styleOuter = {
            position: 'relative',
            display: 'block',
            color: 'white',
            overflow: 'hidden',
        }
        const styleMeta = {
            position: 'absolute',
            display: 'flex',
            alignItems: 'center',
            justifyContent: 'center',
            height: '100%',
            width: '100%',
            fontFamily: 'monospace',
            fontSize: 'xxx-large',
        }
        const styleVideo = {
            objectFit: Settings.get('blackBars') ? 'contain' : 'cover',
            width: '100%',
            height: '100%',
            transform: username === State.username ? 'scaleX(-1)' : 'scaleX(1)',
        }
        return m('.video-container', {style: styleOuter},
            m('.video-info', {style: styleMeta},
                m('.username', username),
            ),
            m('video[playsinline][autoplay]', {
                style: styleVideo,
                oncreate: this.setUp(username),
                onremove: this.tearDown(username),
            }),
        )
    },
}
const StreamContainer = {
    videoOn: true,
    audioOn: true,
    view() {
        const dims = [
            Math.floor((1 + Math.sqrt(4 * State.online.length - 3)) / 2),
            Math.ceil(Math.sqrt(State.online.length)),
        ].map(n => Array(n || 1).fill('1fr').join(' '))

        if(innerHeight > innerWidth) dims.reverse()
        const style = {
            backgroundColor: 'black',
            height: '100%',
            overflow: 'hidden',
            display: 'grid',
            gridTemplateRows: dims[0],
            gridTemplateColumns: dims[1],
        }
        return m('.videos', {style},
            State.online.map(username => m(Video, {key: username, username}))
        )
    },
}
const signalPeerStop = (username) => {
    signal({kind: 'peerInfo', value: {type: 'stop'}, source: username})
}
addEventListener('pagehide', () => State.online.forEach(signalPeerStop))
addEventListener('logout', () => State.online.forEach(signalPeerStop))
addEventListener('load', () => {
    Headers.push([VideoConfig])
    Apps.push([StreamContainer, {key: 'stream-container'}])
})
